/* Arduino GreenHouse + Loxone 3.86 + 4 line LCD
  192.168.10.101
  port - 55059
  killeriq@gmail.com
*/

#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <DHT.h>
#include <LiquidCrystal_I2C.h>
#include <NewPing.h>

LiquidCrystal_I2C lcd(0x3F, 20, 4);

// PIN Settings

#define RELAY1 2
#define RELAY2 3
#define RELAY3 5
#define RELAY4 6
#define DHT1PIN 4
#define DHT2PIN A1
#define Button1 A0

//JSN-SR04T Ultrazvukovy senzor
#define pinTrigger    8
#define pinEcho       9
#define maxdist       250

NewPing sonar(pinTrigger, pinEcho, maxdist);

//DHT
#define DHT1TYPE DHT11   // DHT 11
#define DHT2TYPE DHT22   // DHT 22  (AM2302)
DHT dht1(DHT1PIN, DHT1TYPE); //// Initialize DHT sensor for normal 16mhz Arduino
DHT dht2(DHT2PIN, DHT2TYPE); //// Initialize DHT sensor for normal 16mhz Arduino

int pocitadlo = 0;
int pocitadloBacklight;
int pocitadloRadar = 0;
int r1, r2, r3, r4; // Relay cele cisla
int t1, h1; // cele cisla - int
float t2, h2; // desatinne cisla - float
// int t, h; // cele cisla

// Network Settings
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xFE };
IPAddress ip(192, 168, 10, 111);
IPAddress gateway(192, 168, 10, 1);
IPAddress subnet(255, 255, 255, 0);

// Local UDP port to listen on
unsigned int localPort = 9000;

// Recipient IP - Loxone
IPAddress RecipientIP(192, 168, 10, 100);

// Recipient UDP Port
unsigned int RecipientPort = 55059;

// Flag Buttonstate
char Buttonstate1 = 0;

/*// Flag Buttonstate
  char Buttonstate1 = 0;
  char Buttonstate2 = 0;
  char Buttonstate3 = 0;
  char Buttonstate4 = 0;
*/

String sensorID1 = "H01"; // nadefinuje nazov hodnoty Humidity DHT21
String sensorID2 = "T01"; // nadefinuje nazov hodnoty Temperature DHT21
String sensorID3 = "H02"; // nadefinuje nazov hodnoty Humidity DHT22
String sensorID4 = "T02"; // nadefinuje nazov hodnoty Temperature DHT22
String RelayID1 = "R01"; // nadefinuje nazov hodnoty Relay1State
String RelayID2 = "R02"; // nadefinuje nazov hodnoty Relay1State
String RelayID3 = "R03"; // nadefinuje nazov hodnoty Relay1State
String RelayID4 = "R04"; // nadefinuje nazov hodnoty Relay1State

// buffers for receiving and sending data
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];

// An EthernetUDP instance to let us send and receive packets over UDP
EthernetUDP Udp;

void setup() {
  // for debug only
  Serial.begin(9600);
  Serial.print("test");

  // DHT begin
  dht1.begin();
  dht2.begin();
  
  lcd.clear();
  lcd.init();// inicializuje displej
  lcd.backlight(); // zapne podsvietenie
//  lcd.setCursor(0, 0);
  lcd.print("GreenHouse 3.85");
  lcd.setCursor(0, 1);
  lcd.print("loading...");
  
  // start Ethernet
  Ethernet.begin(mac, ip);

  // Start UDP
  Udp.begin(localPort);

  // Button1
  pinMode(Button1, OUTPUT); // pre Analog - INPUT pre digital
  digitalWrite(Button1, HIGH);

  /*
    // Button2
    pinMode(Button2, INPUT);
    digitalWrite(Button2, HIGH);
  */
  digitalWrite(RELAY1, HIGH);
  digitalWrite(RELAY2, HIGH);
  digitalWrite(RELAY3, HIGH);
  digitalWrite(RELAY4, HIGH);

  // Nastavi PIN ako DIGITALNY vystup
  pinMode(RELAY1, OUTPUT);   // RELAY1 on Pin2
  pinMode(RELAY2, OUTPUT);   // RELAY2 on Pin3
  pinMode(RELAY3, OUTPUT);   // RELAY3 on Pin4
  pinMode(RELAY4, OUTPUT);   // RELAY4 on Pin5

  // send UDP Ready
  sendUDP("UDP Ready");

}

void loop()
{
  checkUDP();
  // int S1;
  int SensorValue1, SensorValue2, SensorValue3, SensorValue4;
  int r1, r2, r3, r4;
  int RE1, RE2;
  int distance;   // načtení vzdálenosti v centimetrech do vytvořené proměnné vzdalenost

  // Button1 (Send)  // Ked stlacim 1x vypne display a ked este raz tak zapne, po 60s sa vypne sam
  if (digitalRead(Button1) == HIGH && Buttonstate1 == 0) // podmienka && obidve hodnoty spolu (ked stlacim tlacitko a bolo vypnute)
  
  { sendUDP("020"), Serial.println("UDP:020 "); 
    lcd.backlight(), Serial.print("backlight ON ");
    pocitadloBacklight = 60;   //tu si skus experimentalne pomenit pocet cykov, pocas ktorych ma byt zapnuty backlight, vyssie cislo - dlhsi cas
    Buttonstate1 = 1;
  }
  if (digitalRead(Button1) == LOW && Buttonstate1 == 1 )
  {
    Buttonstate1 = 0;
  }
  if (pocitadloBacklight > 0) pocitadloBacklight--;
  if (pocitadloBacklight == 0) lcd.noBacklight();
  //  S1 = digitalRead(Button1);
  //  Serial.print("BT:"), Serial.print(S1);

 /* RE1 = digitalRead(RELAY1);
  RE2 = digitalRead(RELAY2);
  Serial.print("R1:");
  Serial.println(RE1);
  Serial.print("R2:");
  Serial.println(RE2);
   delay(150);
*/
    // RELAY1 (Receive) - U Relay 1/2 pridana podmienka ked je R1 zapnute tak nezapni R2 a naopak
    // je to z dovodu ze niekedy Loxone nechyti prikaz alebo sa "sekne" cize ak by ste mali Ventil doprava/dolava tak ho asi odpalite, cize som to pre istotu osetrit takto.
  if (!strcmp(packetBuffer, "020") && (digitalRead(RELAY2) == HIGH)) digitalWrite(RELAY1, LOW);    //Relay 1 switches to NC
  if (!strcmp(packetBuffer, "021")) digitalWrite(RELAY1, HIGH);  //Relay 1 switches to NO
  
  // RELAY2 (Receive)
  if (!strcmp(packetBuffer, "022") && (digitalRead(RELAY1) == HIGH)) digitalWrite(RELAY2, LOW);
  if (!strcmp(packetBuffer, "023")) digitalWrite(RELAY2, HIGH); 
  
  // RELAY3 (Receive)
  if (!strcmp(packetBuffer, "024")) digitalWrite(RELAY3, LOW);
  if (!strcmp(packetBuffer, "025")) digitalWrite(RELAY3, HIGH);

  // RELAY4 (Receive)
  if (!strcmp(packetBuffer, "026"))  digitalWrite(RELAY4, LOW);
  if (!strcmp(packetBuffer, "027"))  digitalWrite(RELAY4, HIGH);

  r1 = digitalRead(RELAY1);
  /*   Serial.print("RELAY1: ");
     Serial.print(r1);
     Serial.print("\t");   */
  r2 = digitalRead(RELAY2);
  /*    Serial.print("RELAY2: ");
      Serial.println(r2);*/
  r3 = digitalRead(RELAY3);
  /*    Serial.print("RELAY3: ");
      Serial.print(r3);
      Serial.print("\t");    */
  r4 = digitalRead(RELAY4);
  /*    Serial.print("RELAY4: ");
      Serial.print(r4);*/

  sendUDP ("R01", r1); // alebo nepriamo sendUDP(sensorID1,SensorValue1)
  sendUDP ("R02", r2); // alebo nepriamo sendUDP(sensorID1,SensorValue1)
  sendUDP ("R03", r3); // alebo nepriamo sendUDP(sensorID1,SensorValue1)
  sendUDP ("R04", r4); // alebo nepriamo sendUDP(sensorID1,SensorValue1)

  delay(50);

  // JSN-SR04T
  if (pocitadloRadar == 20)    //tu tiez mozes poexperimentovat s poctom cyklov, cim vysie cislo, tym vacsi odstup medzi meraniami
  {
    distance = sonar.ping_cm();
    Serial.print("Vzdialenost: ");
    Serial.print(distance);   // ked to tu skape, nepusta sa do dalsieho merania
    delay(50);
    if (distance > 0) {
      distance = 0;
      for (int i = 0; i < 5; i++) {
        distance += sonar.ping_cm();
        delay(50);
      }
      distance = distance / 5;
      Serial.print("Vzdalenost medzi senzorom a predmetom je ");
      Serial.print(distance);
      Serial.println(" cm.");
      sendUDP ("D01", distance); // posle udaj cez UDP
      lcd.setCursor(0, 2);
      lcd.print("Distance: ");
      lcd.print(distance);
      lcd.print(" cm");
    }
    else {
      Serial.println("Vzdalenost mezi senzorem a predmetem je mimo merici rozsah.");
      delay(50);
    }
    pocitadloRadar = 0;
  }
  


  if (pocitadlo == 40) {
    h1 = dht1.readHumidity(); // nacita hodnotu vlhkosti
    t1 = dht1.readTemperature(); // nacita hodnotu teploty
    Serial.println(t1);
    Serial.println(h1);
    h2 = dht2.readHumidity(); // nacita hodnotu vlhkosti
    t2 = dht2.readTemperature(); // nacita hodnotu teploty
    Serial.println(t2);
    Serial.println(h2);

    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("T1:");
    lcd.print(t1);
    lcd.print("C");
    //lcd.setCursor(0, 1);
    lcd.print(" H1:");
    lcd.print(h1);
    lcd.print("%");
    lcd.setCursor(0, 1);
    lcd.print("T2:");
    lcd.print(t2);
    lcd.print("C");
    lcd.print(" H2:");
    lcd.print(h2);
    lcd.print("%");

    // SensorValue1 = h1; // hodnotu znasobi 10x aby poslalo aj desatinne cisla
    // SensorValue2 = t1;  // int / float same result
    sendUDP ("H01", h1); // posle string aj s nameranou hodnotou, priamo sendUDP("H01",SensorValue1)
    sendUDP ("T01", t1); // alebo nepriamo sendUDP(sensorID1,SensorValue1)

    SensorValue3 = 10 * h2; // hodnotu znasobi 10x aby poslalo aj desatinne cisla
    SensorValue4 = 10 * t2;  // int / float same result
    sendUDP ("H02", SensorValue3); // posle string aj s nameranou hodnotou, priamo sendUDP("H01",SensorValue1)
    sendUDP ("T02", SensorValue4); // alebo nepriamo sendUDP(sensorID1,SensorValue1)

    pocitadlo = 0;
  }
  pocitadlo++;
  pocitadloRadar++;
  if (pocitadloRadar == 4000) pocitadloRadar=0;   //tu tiez mozes poexperimentovat s poctom cyklov, cim vysie cislo, tym vacsi odstup medzi meraniami
}

// Function to check for new UDP packets
void checkUDP()
{
  // if there's data available, read a packet
  int packetSize = Udp.parsePacket();
  if (packetSize)
  {
    // read the packet into packetBufffer
    Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);

    // for debug only
    // write packetBuffer to serial
    Serial.print("Packet Content: ");
    Serial.println(packetBuffer);

    // send "ACK" as reply
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    Udp.write("ACK");
    Udp.endPacket();
  }
  delay(10);
}

// Function to send SENZOR UDP packets
void sendUDP (String sensorID, int sensorValue) // posle vsetky hodnoty
{
  Udp.beginPacket(RecipientIP, RecipientPort);
  Udp.print(sensorID); // doesnt matter if sensorID or sensorID2 sends values anyway
  //  Udp.print(RelayID1); // doesnt matter if sensorID or sensorID2 sends values anyway
  Udp.print(sensorValue); // sends values anyway
  //  Udp.print(RelayState1); // sends values anyway
  Udp.endPacket();
  delay(10);
}

// Function to send UDP packets
void sendUDP(String text)
{
  Udp.beginPacket(RecipientIP, RecipientPort);
  // Udp.write("Test");
  Udp.print(text);
  Udp.endPacket();
  delay(10);
}
